-- Affinity_Yield_Utilities
-- Author: HandyVac
-- DateCreated: 2/19/2015 8:18:55 PM
--------------------------------------------------------------

---------------------------------------------------------------------------------
--                          Cache Database Lookups                             --
---------------------------------------------------------------------------------

-- cache relevant data from game database queries, so it can be accessed quickly later on.

hstructure ItemAffinYieldInfo
	Supremacy : number;
	Harmony : number;
	Purity : number;
end

hstructure AffinYieldItemPerkInfo
	Type : string;
	PerkID : number;
	FlatSupremacy : number;
	FlatHarmony : number;
	FlatPurity : number;
	PercentSupremacy : number;
	PercentHarmony : number;
	PercentPurity : number;	
end

--[[
hstructure AffinYieldImprovementPerkInfo
	ImprovementType : string;
	PerkID : number;
	FlatSupremacy : number;
	FlatHarmony : number;
	FlatPurity : number;
end
--]]

-- Cache Building flat yields and yield modifiers. Index by Type string of the building.
CachedBuildingAffinYields = {};
CachedBuildingAffinYieldMods = {};

--print("Caching building affin yield data");
-- iterate through all buildings in the database
for row in GameInfo.Buildings() do
	-- if the building has an affin flat yield or percent yield modifier, grab it. Otherwise use the value 0.
	local iSupremacy = 0;
	local iHarmony = 0;
	local iPurity = 0;
	local iSupremacyMod = 0;
	local iHarmonyMod = 0;
	local iPurityMod = 0;
	
	if row.SupremacyPerTurn then iSupremacy = row.SupremacyPerTurn end;
	if row.HarmonyPerTurn then iHarmony = row.HarmonyPerTurn end;
	if row.PurityPerTurn then iPurity = row.PurityPerTurn end;
	if row.SupremacyModifier then iSupremacyMod = row.SupremacyModifier end;
	if row.HarmonyModifier then iHarmonyMod = row.HarmonyModifier end;
	if row.PurityModifier then iPurityMod = row.PurityModifier end;

	-- add an entry for this building to the cache tables
	CachedBuildingAffinYields[row.Type] = hmake ItemAffinYieldInfo { Supremacy = iSupremacy, Harmony = iHarmony, Purity = iPurity };
	CachedBuildingAffinYieldMods[row.Type] = hmake ItemAffinYieldInfo { Supremacy = iSupremacyMod, Harmony = iHarmonyMod, Purity = iPurityMod };
end

	--print("TESTING Relic yields ( " .. CachedBuildingAffinYields["BUILDING_RELIC"].Purity .. " ) Purity");


-- Cache Improvement flat yields. Index by Type string of the improvement.
CachedImprovementAffinYields = {};
-- print("Caching improvement affin yield data");

-- iterate through all improvements in the database
for row in GameInfo.Improvements() do
	-- set default values to 0.
	local iSupremacy = 0;
	local iHarmony = 0;
	local iPurity = 0;

	-- if the improvement has an actual value for one of these variables, grab it.
	if row.SupremacyPerTurn then iSupremacy	= row.SupremacyPerTurn	end;
	if row.HarmonyPerTurn then iHarmony = row.HarmonyPerTurn end;
	if row.PurityPerTurn then iPurity = row.PurityPerTurn end;

	-- add an entry for this improvement in the cache table
	CachedImprovementAffinYields[row.Type] = hmake ItemAffinYieldInfo { Supremacy = iSupremacy, Harmony = iHarmony, Purity = iPurity };
end
--print("TESTING Petrol Well yields ( " .. CachedImprovementAffinYields["IMPROVEMENT_PETROL_WELL"].Purity .. " ) Purity");

--[[
-- Cache Terrain Feature flat yields. Index by Type string of the feature.
CachedFeatureAffinYields = {};
print("caching terrain feature affin yield data");

-- iterate through all terrain features in the database
for row in GameInfo.Features() do
	-- set default values
	local iSupremacy = 0;
	local iHarmony = 0;
	local iPurity = 0;
	
	-- if this feature has an affin yield associated with it, grab that value
	if GameInfo.Feature_AffinityYields[row.Type] ~= nil then 
		iSupremacy = GameInfo.Feature_AffinityYields[row.Type].FlatSupremacy;
		iHarmony = GameInfo.Feature_AffinityYields[row.Type].FlatHarmony;
		iPurity = GameInfo.Feature_AffinityYields[row.Type].FlatPurity;
	end
	
	-- create an entry in the cache for this feature
		CachedFeatureAffinYields[row.Type] = hmake ItemAffinYieldInfo { Supremacy = iSupremacy, Harmony = iHarmony, Purity = iPurity };
end
--]]


-- Cache Perks that affect affinity yields. Index by an ID number.

--  Cache BUILDING perks
CachedAffinYieldBuildingPerks = {};

--print("Caching perk data for affin yields");
-- initialise a counter to serve as the ID of this cache item
local counter = 0;
-- iterate through all building-affecting perks in the database
for perk in GameInfo.PlayerPerks_GeneralBuildingEffects() do
	-- set default values
	local iFlatSupremacy = 0;
	local iFlatHarmony = 0;
	local iFlatPurity = 0;
	local iPercentSupremacy = 0;
	local iPercentHarmony = 0;
	local iPercentPurity = 0;
	
	if GameInfo.PlayerPerks[perk.PlayerPerkType] ~= nil then
		-- grab values from the database
		local iPerkID = GameInfo.PlayerPerks[perk.PlayerPerkType].ID;

		if perk.FlatSupremacy then iFlatSupremacy = perk.FlatSupremacy end;
		if perk.FlatHarmony then iFlatHarmony = perk.FlatHarmony end
		if perk.FlatPurity then iFlatPurity = perk.FlatPurity end;
						
		if perk.PercentSupremacy then iPercentSupremacy = perk.PercentSupremacy end;
		if perk.PercentHarmony then iPercentHarmony = perk.PercentHarmony end;
		if perk.PercentPurity then iPercentPurity = perk.PercentPurity end;

		-- perks table refers to a class of building. Iterate through up all building TYPES of this CLASS and add an identical entry for each of them to our cache.
		for building in GameInfo.Buildings() do
			-- is the building of the class we're interested in?
			if building.BuildingClass == perk.BuildingClassType then 
				-- make an entry in the cache for this building type
				CachedAffinYieldBuildingPerks[counter] = 							hmake AffinYieldItemPerkInfo	{
																														Type = building.Type, 
																														PerkID = GameInfo.PlayerPerks[perk.PlayerPerkType].ID,
																														FlatSupremacy =  iFlatSupremacy,
																														FlatHarmony = iFlatHarmony, 
																														FlatPurity = iFlatPurity,
																														PercentSupremacy = iPercentSupremacy, 
																														PercentHarmony = iPercentHarmony, 
																														PercentPurity = iPercentPurity
																													};
			end
			counter = counter +1;
		end
	end
end
-- delete counter
counter = nil;

-- Cache IMPROVEMENT perks
CachedAffinYieldImprovementPerks = {};

--print("caching improvement perks");
-- initialise a counter to serve as the ID of cache items
local counter = 0;
-- iterate through all perks that affect affinity-yields of improvements
for perk in GameInfo.PlayerPerks_ImprovementAffinityYieldEffects() do
	--print("caching perk: " .. perk.PlayerPerkType);
	
	-- set default values
	local iFlatSupremacy = 0;
	local iFlatHarmony = 0;
	local iFlatPurity = 0;
	
	-- grab values from the database
	if perk.FlatSupremacy then iFlatSupremacy = perk.FlatSupremacy end;
	if perk.FlatHarmony then iFlatHarmony = perk.FlatHarmony end
	if perk.FlatPurity then iFlatPurity = perk.FlatPurity end;
	
	-- make an entry in the cache for this perk
	CachedAffinYieldImprovementPerks[counter] = 							hmake AffinYieldItemPerkInfo	{
																												Type = perk.ImprovementType, 
																												PerkID = GameInfo.PlayerPerks[perk.PlayerPerkType].ID,
																												FlatSupremacy = iFlatSupremacy, 
																												FlatHarmony = iFlatHarmony, 
																												FlatPurity = iFlatPurity,
																												PercentSupremacy = 0, 
																												PercentHarmony = 0, 
																												PercentPurity = 0
																											};
	counter = counter + 1;
end
--print("finished caching improvement perks");
-- delete the counter
counter = nil;

-- Cache FEATURE perks
CachedAffinYieldFeaturePerks = {};
--print("caching terrain feature affin yield perks");

-- initialise a counter to serve as the ID of cache items
local counter = 0;
-- iterate through all perks that affect affinity-yields of improvements
for perk in GameInfo.PlayerPerks_FeatureAffinityYieldEffects() do
	--print("caching perk: " .. perk.PlayerPerkType);
	
	-- set default values
	local iFlatSupremacy = 0;
	local iFlatHarmony = 0;
	local iFlatPurity = 0;
	
	-- grab values from the database
	if perk.FlatSupremacy then iFlatSupremacy = perk.FlatSupremacy end;
	if perk.FlatHarmony then iFlatHarmony = perk.FlatHarmony end
	if perk.FlatPurity then iFlatPurity = perk.FlatPurity end;
	
	-- make an entry in the cache for this perk
	CachedAffinYieldFeaturePerks[counter] = 									hmake AffinYieldItemPerkInfo	{
																													Type = perk.FeatureType, 
																													PerkID = GameInfo.PlayerPerks[perk.PlayerPerkType].ID,
																													FlatSupremacy = iFlatSupremacy, 
																													FlatHarmony = iFlatHarmony, 
																													FlatPurity = iFlatPurity,
																													PercentSupremacy = 0, 
																													PercentHarmony = 0, 
																													PercentPurity = 0
																												};
	counter = counter + 1;
end
--print("finished caching terrain feature perks");
-- delete the counter
counter = nil;





------------------------------------------------------------------------------------
--                              Utility Functions                                 --
------------------------------------------------------------------------------------

-- Calculates total affinity yields per turn for a given player, and returns them as a table
function GetTotalAffinityYieldsForPlayer(PlayerID)
	local affinityTotals = { Supremacy = 0, Harmony = 0, Purity = 0 };
	local pPlayer = Players[PlayerID];
	
	--print("Getting total affinity yields for player " .. PlayerID);
	
	-- FROM CITIES
	-- Iterate through all the player's cities and add that city's yields to the total
	for pCity in pPlayer:Cities() do
		local affinityYieldsFromThisCity = GetAffinityYieldsFromCity(pCity);

		affinityTotals.Supremacy 	= affinityTotals.Supremacy 	+ affinityYieldsFromThisCity.Supremacy;
		affinityTotals.Harmony		= affinityTotals.Harmony	+ affinityYieldsFromThisCity.Harmony;
		affinityTotals.Purity		= affinityTotals.Purity		+ affinityYieldsFromThisCity.Purity;
	end
	
	
	-- FROM ANY OTHER SOURCES
	
	return affinityTotals;
end


-- Calculates affinity yields generated by a given city, and returns them as a table
function GetAffinityYieldsFromCity(pCity)
	local affinityFromCity = { Supremacy = 0, Harmony = 0, Purity = 0, Dominant = "" };
	local affinityModifiers = { Supremacy = 0, Harmony = 0, Purity = 0 };
	
	--print( "Checking city " .. pCity:GetName() );

	-- FROM WORKED PLOTS
	
	-- Iterate through all plots within a square 3 x 3 grid around the city
	for gridX = -3, 3 do
		for gridY = -3, 3 do
			-- Check whether this plot is within workable range (i.e. a 3 tile *circle* from the city)
			local Plot = Map.PlotXYWithRangeCheck(pCity:GetX(), pCity:GetY(), gridX, gridY, 3);
			if Plot then
				-- check whether the city is actually working this plot
				if Plot:IsBeingWorked() and Plot:GetWorkingCity() == pCity then
					-- Grab affinity yields from this plot and add them to the city's total
					local affinityFromThisPlot =  GetAffinityYieldsFromPlot(Plot);
					
					affinityFromCity.Supremacy	=	affinityFromCity.Supremacy	+ affinityFromThisPlot.Supremacy;
					affinityFromCity.Harmony	=	affinityFromCity.Harmony	+ affinityFromThisPlot.Harmony;
					affinityFromCity.Purity		=	affinityFromCity.Purity		+ affinityFromThisPlot.Purity;

					--print("worked plot: ( " .. affinityFromThisPlot.Supremacy .. "S, " .. affinityFromThisPlot.Harmony .. "H, " .. affinityFromThisPlot.Purity .. "P )");
				end
			end
		end
	end


	-- FROM BUILDINGS
	
	-- loop through all possible types of building, and check if the city has one
	for pBuilding in GameInfo.Buildings() do
			if pCity:IsHasBuilding(pBuilding.ID) then
				-- Grab affinity yields from this building and add them to the city's total
				local affinityFromThisBuilding = GetAffinityYieldsFromBuilding(pBuilding.Type);
				
				affinityFromCity.Supremacy		=	affinityFromCity.Supremacy	+ affinityFromThisBuilding.Supremacy;
				affinityFromCity.Harmony		=	affinityFromCity.Harmony	+ affinityFromThisBuilding.Harmony;
				affinityFromCity.Purity			=	affinityFromCity.Purity		+ affinityFromThisBuilding.Purity;
				
				-- also grab percentage modifiers from this building, and add them to the city's total of percentage modifiers
				local affinityModifiersFromThisBuilding = GetAffinityYieldModifiersFromBuilding(pBuilding.Type);

				affinityModifiers.Supremacy		= affinityModifiers.Supremacy 	+ affinityModifiersFromThisBuilding.Supremacy;
				affinityModifiers.Harmony		= affinityModifiers.Harmony 	+ affinityModifiersFromThisBuilding.Harmony;
				affinityModifiers.Purity		= affinityModifiers.Purity 		+ affinityModifiersFromThisBuilding.Purity;

				--print(Locale.ConvertTextKey(pBuilding.Description) .. ": ( " .. affinityFromCity.Supremacy .. "S, " .. affinityFromCity.Harmony .. "H, " .. affinityFromCity.Purity .. "P ) ( " .. affinityModifiers.Supremacy .. "%S, " .. affinityModifiers.Harmony .. "%H, " .. affinityModifiers.Purity .. "%P )");
			end
	end
	
	-- FROM OTHER SOURCES
	
	
	
	-- APPLY PERCENTAGE MODIFIERS
	--print("Applying modifiers to city yield: S+" .. affinityModifiers.Supremacy .. "%, H+" .. affinityModifiers.Harmony .. "%, P+" .. affinityModifiers.Purity .. "%");
	affinityFromCity.Supremacy	=	affinityFromCity.Supremacy 	/100 * (100 + affinityModifiers.Supremacy);
	affinityFromCity.Harmony	=	affinityFromCity.Harmony	/100 * (100 + affinityModifiers.Harmony);
	affinityFromCity.Purity		=	affinityFromCity.Purity		/100 * (100 + affinityModifiers.Purity);

	--[[
	-- DETERMINE LOCAL DOMINANT AFFINITY
	local cityDominantAffinity = nil;
	if affinityFromCity.Supremacy > affinityFromCity.Harmony and affinityFromCity.Supremacy > affinityFromCity.Purity then
		print("City's dominant affinity is Supremacy, halving other yields");
		affinityFromCity.Harmony = affinityFromCity.Harmony / 2;
		affinityFromCity.Purity	= affinityFromCity.Purity / 2;
		affinityFromCity.Dominant = "Supremacy";
	elseif affinityFromCity.Harmony > affinityFromCity.Supremacy and affinityFromCity.Harmony > affinityFromCity.Purity then
		print("City's dominant affinity is Harmony, halving other yields");
		affinityFromCity.Supremacy = affinityFromCity.Supremacy / 2;
		affinityFromCity.Purity = affinityFromCity.Purity / 2;
		affinityFromCity.Dominant = "Harmony";
	elseif affinityFromCity.Purity > affinityFromCity.Supremacy and affinityFromCity.Purity > affinityFromCity.Harmony then
		print("City's dominant affinity is Purity, halving other yields");
		affinityFromCity.Supremacy = affinityFromCity.Supremacy / 2;
		affinityFromCity.Harmony = affinityFromCity.Harmony / 2;
		affinityFromCity.Dominant = "Purity";
	else
		print("City has no dominant affinity");
	end
	--]]

	-- return the added up totals
	--print(pCity:GetName() .. " : " .. affinityFromCity.Supremacy .. "S " .. affinityFromCity.Harmony .. "H " .. affinityFromCity.Purity .. "P");
	return affinityFromCity;
end;

-- calculates affinity yields from all buildings owned by this city, and returns them as a table
function GetAffinityYieldsFromCityBuildings(pCity)
local affinityTotals =  { Supremacy = 0, Harmony = 0, Purity = 0 };

-- loop through all possible types of building, and check if the city has one
	for pBuilding in GameInfo.Buildings() do
		
		if pCity:IsHasBuilding(pBuilding.ID) then
			-- Grab affinity yields from this building and add them to the city's total
			local affinityFromThisBuilding = GetAffinityYieldsFromBuilding(pBuilding.Type);
				
			affinityTotals.Supremacy		=	affinityTotals.Supremacy	+ affinityFromThisBuilding.Supremacy;
			affinityTotals.Harmony			=	affinityTotals.Harmony		+ affinityFromThisBuilding.Harmony;
			affinityTotals.Purity			=	affinityTotals.Purity				+ affinityFromThisBuilding.Purity;
			
		end
	end
	
	return affinityTotals;
end

-- calculates affinity yield-modifiers from all the buildings this city has, and returns them as a table
function GetAffinityYieldModsFromCityBuildings(pCity)
	local affinityModifiers =  { Supremacy = 0, Harmony = 0, Purity = 0 };

	for pBuilding in GameInfo.Buildings() do
			if pCity:IsHasBuilding(pBuilding.ID) then
						
				--  grab percentage modifiers from this building, and add them to the city's total of percentage modifiers
				local affinityModifiersFromThisBuilding = GetAffinityYieldModifiersFromBuilding(pBuilding.Type);

				affinityModifiers.Supremacy		= affinityModifiers.Supremacy 	+ affinityModifiersFromThisBuilding.Supremacy;
				affinityModifiers.Harmony			= affinityModifiers.Harmony 		+ affinityModifiersFromThisBuilding.Harmony;
				affinityModifiers.Purity					= affinityModifiers.Purity 				+ affinityModifiersFromThisBuilding.Purity;

			end
	end
	
	return affinityModifiers;
end

-- calculates affinity yields from all plots worked by this city, and returns them as a table.
function GetAffinityYieldsFromCityPlots(pCity)
	local affinityTotals = { Supremacy = 0, Harmony = 0, Purity = 0 };

-- Iterate through all plots within a square 3 x 3 grid around the city
	for gridX = -3, 3 do
		for gridY = -3, 3 do
			-- Check whether this plot is within workable range (i.e. a 3 tile *circle* from the city)
			local Plot = Map.PlotXYWithRangeCheck(pCity:GetX(), pCity:GetY(), gridX, gridY, 3);
			if Plot then
				-- check whether the city is actually working this plot
				if Plot:IsBeingWorked() and Plot:GetWorkingCity() == pCity then
					-- Grab affinity yields from this plot and add them to the city's total
					local affinityFromThisPlot =  GetAffinityYieldsFromPlot(Plot);
					
					affinityTotals.Supremacy	=	affinityTotals.Supremacy	+ affinityFromThisPlot.Supremacy;
					affinityTotals.Harmony		=	affinityTotals.Harmony		+ affinityFromThisPlot.Harmony;
					affinityTotals.Purity				=	affinityTotals.Purity				+ affinityFromThisPlot.Purity;

					--print("worked plot: ( " .. affinityFromThisPlot.Supremacy .. "S, " .. affinityFromThisPlot.Harmony .. "H, " .. affinityFromThisPlot.Purity .. "P )");
				end
			end
		end
	end
	
	return affinityTotals;
end



-- Calculates affinity yields from a given plot, and returns them as a table.
function GetAffinityYieldsFromPlot(Plot)
	local PlotAffinityYields = { Supremacy = 0, Harmony = 0, Purity = 0 };
		
	--print("* Checking a plot")
	-- FROM TERRAIN FEATURES
	-- check what type of feature is on this plot
	local featureID = Plot:GetFeatureType() ;
	
	if featureID == -1 then 
		--print("no terrain feature present");
	else
		--print("plot has a " .. Locale.ConvertTextKey(GameInfo.Features[featureID].Description) );
		
		--grab affinity yield data for this improvement
		local affinityFromFeature = GetAffinityYieldsFromFeature(GameInfo.Features[featureID].Type);
		
		-- add to the plot's total
		PlotAffinityYields.Supremacy 		= PlotAffinityYields.Supremacy 		+ affinityFromFeature.Supremacy;
		PlotAffinityYields.Harmony 			= PlotAffinityYields.Harmony 			+ affinityFromFeature.Harmony;
		PlotAffinityYields.Purity 				= PlotAffinityYields.Purity 				+ affinityFromFeature.Purity;			
	end
	
	
	-- FROM IMPROVEMENTS

	-- Check what type of improvement is on this plot (-1 signifies no improvement)
	local improvementID = Plot:GetImprovementType();
	if improvementID == -1 then
		--print("plot is unimproved" );
	else 	
		--print("plot has a " .. Locale.ConvertTextKey(GameInfo.Improvements[improvementID].Description) )
		
		-- grab affinity yields from this improvement
		local affinityFromImprovement = GetAffinityYieldsFromImprovement(GameInfo.Improvements[improvementID].Type);
		
		-- add to the plot's total
		PlotAffinityYields.Supremacy 		= PlotAffinityYields.Supremacy 	+ affinityFromImprovement.Supremacy;
		PlotAffinityYields.Harmony 			= PlotAffinityYields.Harmony 	+ affinityFromImprovement.Harmony;
		PlotAffinityYields.Purity 			= PlotAffinityYields.Purity 	+ affinityFromImprovement.Purity;			
	end
		
	
	-- FROM MIASMA
	-- miasma is a special case, it is not handled by the game as a regular terrain feature.

	-- does the plot have miasma?
	if Plot:HasMiasma() then
		--print("this plot has miasma");
		-- iterate through all perks which affect affin yields from features
		for perk, ItemAffinYieldInfo in pairs(CachedAffinYieldFeaturePerks) do
			-- does this perk affect miasma?
			if ItemAffinYieldInfo.Type ==  "FEATURE_MIASMA" then
				--print("a perk exists which affects affin yields from miasma");
				-- does the player have this perk?
				local pActivePlayer = Players[Game.GetActivePlayer()];

				if pActivePlayer:HasPerk(ItemAffinYieldInfo.PerkID) then
					--print("and the active player owns it")
					-- add the perk's affects to the yield total
					PlotAffinityYields.Supremacy	= PlotAffinityYields.Supremacy	+ ItemAffinYieldInfo.FlatSupremacy;
					PlotAffinityYields.Harmony		= PlotAffinityYields.Harmony	+ ItemAffinYieldInfo.FlatHarmony;
					PlotAffinityYields.Purity		= PlotAffinityYields.Purity		+ ItemAffinYieldInfo.FlatPurity;
				end
			end
		end
	end
	
	-- FROM ANY OTHER SOURCES


	--print("( " .. PlotAffinityYields.Supremacy .. "S, " .. PlotAffinityYields.Harmony .. "H, " .. PlotAffinityYields.Purity .. "P )");
	return PlotAffinityYields;
end

-- Calculates what affinity yields from plot *would be* if a given build order were performed on it
function GetAffinityYieldsWithBuild(pPlot, pBuild)
	--print("Getting hypothetical yield if we build a " .. pBuild.Description);

	-- get current yield of plot
	local AffinityYield = GetAffinityYieldsFromPlot(pPlot);

	-- IMPROVEMENTS
	-- Will the build order create an improvement?
	if pBuild.ImprovementType ~= nil then
		--print("hypothetical build will produce a " .. pBuild.ImprovementType);
		-- add that improvement's affinity yields to the total
		local affinityFromNewImprovement = GetAffinityYieldsFromImprovement(pBuild.ImprovementType);

		AffinityYield.Supremacy	= AffinityYield.Supremacy	+ affinityFromNewImprovement.Supremacy;
		AffinityYield.Harmony		= AffinityYield.Harmony		+ affinityFromNewImprovement.Harmony;
		AffinityYield.Purity			= AffinityYield.Purity			+ affinityFromNewImprovement.Purity;

		-- Is there already an improvement on this plot, which would be replaced?
		local iPresentImprovementID = pPlot:GetImprovementType();
		if iPresentImprovementID ~= -1 then
			--print("hypothetical build will remove a " .. GameInfo.Improvements[iPresentImprovementID].Description );
			-- subtract that improvement's yields from the total
			local affinityFromOldImprovement = GetAffinityYieldsFromImprovement(GameInfo.Improvements[iPresentImprovementID].Type);

			AffinityYield.Supremacy	= AffinityYield.Supremacy	- affinityFromOldImprovement.Supremacy;
			AffinityYield.Harmony		= AffinityYield.Harmony		- affinityFromOldImprovement.Harmony;
			AffinityYield.Purity			= AffinityYield.Purity			- affinityFromOldImprovement.Purity;
		end
		
	end
	
	-- FEATURES 
	
	-- Will the build add a feature (including miasma)?
	if pBuild.FeatureTypeChange ~= nil then
		-- add that feature's affinity yield to the total
		local affinityFromNewFeature = GetAffinityYieldsFromFeature(pBuild.FeatureTypeChange);
		
		AffinityYield.Supremacy		= AffinityYield.Supremacy	+ affinityFromNewFeature.Supremacy;
		AffinityYield.Harmony			= AffinityYield.Harmony		+ affinityFromNewFeature.Harmony;
		AffinityYield.Purity				= AffinityYield.Purity			+ affinityFromNewFeature.Purity;
		
		-- only one feature may exist at a time (except miasma, but miasma is not noticed by plot:GetFeatureType() ). 
		-- Is there already a feature here (other than miasma) that would be replaced?
		local featureID = pPlot:GetFeatureType() ;
		if featureID ~= -1 then
			-- subtract that feature's affinity yield from the total
			local affinityFromOldFeature = GetAffinityYieldsFromFeature(GameInfo.Features[featureID].Type);
			
			AffinityYield.Supremacy	= AffinityYield.Supremacy	- affinityFromOldFeature.Supremacy;
			AffinityYield.Harmony		= AffinityYield.Harmony		- affinityFromOldFeature.Harmony;
			AffinityYield.Purity			= AffinityYield.Purity			- affinityFromOldFeature.Purity;
		end
		
	end
	
	-- Will the build remove a feature?
	
	-- loop through all builds that interact with features (including miasma)
		for row in GameInfo.BuildsOnFeatures() do
			-- does it remove the feature it's interacting with?
			if row.Remove then
				-- is this the build order we're looking for?
				if row.BuildType == pBuild.Type then	
					-- is the feature in question (except miasma) present on this plot?
					local featureID = pPlot:GetFeatureType();
					if featureID ~= -1 then 
						if row.FeatureType == GameInfo.Features[featureID].Type then
							-- subtract the affinity yields of that feature from the total
							local affinityFromOldFeature = GetAffinityYieldsFromFeature(row.FeatureType);
				
							AffinityYield.Supremacy	= AffinityYield.Supremacy	- affinityFromOldFeature.Supremacy;
							AffinityYield.Harmony		= AffinityYield.Harmony		- affinityFromOldFeature.Harmony;
							AffinityYield.Purity			= AffinityYield.Purity			- affinityFromOldFeature.Purity;					
						end
					end
						-- is the feature in question (if it's miasma) present on this plot?
					if pPlot:HasMiasma()  then
						if row.FeatureType == "FEATURE_MIASMA" then
							-- subtract the miasma's affinity yields from the total
							local affinityFromOldFeature = GetAffinityYieldsFromFeature(row.FeatureType);
							
							AffinityYield.Supremacy	= AffinityYield.Supremacy	- affinityFromOldFeature.Supremacy;
							AffinityYield.Harmony		= AffinityYield.Harmony		- affinityFromOldFeature.Harmony;
							AffinityYield.Purity			= AffinityYield.Purity			- affinityFromOldFeature.Purity;
						end
					end
					
				end
			end
		end
	
	
	
	return AffinityYield;
end


-- Calculates affinity yields from a given improvement type, and returns them as a table.
function GetAffinityYieldsFromImprovement(improvementType)
	local AffinityYields = { Supremacy = 0, Harmony = 0, Purity = 0 };
	--print("getting affin yields for an improvement of type " .. improvementType);
	
	-- Base yields of this improvement type
	
	AffinityYields.Supremacy	= CachedImprovementAffinYields[improvementType].Supremacy;
	AffinityYields.Harmony		= CachedImprovementAffinYields[improvementType].Harmony;
	AffinityYields.Purity				= CachedImprovementAffinYields[improvementType].Purity;
	
	-- Extra yields for this improvement type, from Perks etc.
	
	
	-- iterate through all perks which affect affinity yields of improvements
	for perk, ItemAffinYieldInfo in pairs(CachedAffinYieldImprovementPerks) do
		-- does this perk affect this improvement
		if ItemAffinYieldInfo.Type == improvementType then
			--print("a perk exists which affects this improvement");
			-- does the player have this perk?
			local pActivePlayer = Players[Game.GetActivePlayer()];
			
			if pActivePlayer:HasPerk(ItemAffinYieldInfo.PerkID) then
				--print("and the active player owns it")
				-- add the perk's affects to the yield total
				AffinityYields.Supremacy = AffinityYields.Supremacy + ItemAffinYieldInfo.FlatSupremacy;
				AffinityYields.Harmony = AffinityYields.Harmony + ItemAffinYieldInfo.FlatHarmony;
				AffinityYields.Purity = AffinityYields.Purity + ItemAffinYieldInfo.FlatPurity;
			end
		end
	end


	--print(GameInfo.Improvements[improvementType].Description .. " yields " .. AffinityYields.Supremacy .. "S " .. AffinityYields.Harmony .. "H " .. AffinityYields.Purity .. "P");		
	return AffinityYields;
end

-- Calculates affinity yields from a given terrain feature type, and returns them as a table.
function GetAffinityYieldsFromFeature(featureType)
	local AffinityYields = { Supremacy = 0, Harmony = 0, Purity = 0 };
	--print("getting affin yields for a " .. featureType);
	
	--Base yields for this feature type
	--[[ no features have base affin yields
	AffinityYields.Supremacy	= CachedFeatureAffinYields[featureType].Supremacy;
	AffinityYields.Harmony 		= CachedFeatureAffinYields[featureType].Harmony;
	AffinityYields.Purity 			= CachedFeatureAffinYields[featureType].Purity;
	--]]
	
	-- Extra yields from perks
		
	-- iterate through all perks which affect affin yields from features
	for perk, ItemAffinYieldInfo in pairs(CachedAffinYieldFeaturePerks) do
		-- does this perk affect this feature?
		if ItemAffinYieldInfo.Type == featureType then
			--print("a perk exists which affects this feature");
			-- does the player have this perk?
			local pActivePlayer = Players[Game.GetActivePlayer()];
			
			if pActivePlayer:HasPerk(ItemAffinYieldInfo.PerkID) then
				--print("and the active player owns it")
				-- add the perk's affects to the yield total
				AffinityYields.Supremacy = AffinityYields.Supremacy + ItemAffinYieldInfo.FlatSupremacy;
				AffinityYields.Harmony = AffinityYields.Harmony + ItemAffinYieldInfo.FlatHarmony;
				AffinityYields.Purity = AffinityYields.Purity + ItemAffinYieldInfo.FlatPurity;
			end
		end
	end

	return AffinityYields;
end

-- Calculates affinity yields from a given building type, and returns them as a table
function GetAffinityYieldsFromBuilding(BuildingType)
	local BuildingAffinityYields = { Supremacy = 0, Harmony = 0, Purity = 0 };
	--print("Getting affinity yields for " .. BuildingType);

	-- Base yields of this building type
	BuildingAffinityYields.Supremacy	= CachedBuildingAffinYields[BuildingType].Supremacy;
	BuildingAffinityYields.Harmony		= CachedBuildingAffinYields[BuildingType].Harmony;
	BuildingAffinityYields.Purity		= CachedBuildingAffinYields[BuildingType].Purity;

	-- Extra yields of this building type, from Perks etc.

	-- iterate through all perks which affect affinity yields of buildings
	for perk, ItemAffinYieldInfo in pairs(CachedAffinYieldBuildingPerks) do
		
		-- does this perk affect this building?
		if ItemAffinYieldInfo.Type == BuildingType then
			--print("A perk exists which affects this building");

			-- does the player have this perk?
			local pActivePlayer = Players[Game.GetActivePlayer()];
			

			if pActivePlayer:HasPerk(ItemAffinYieldInfo.PerkID) then
				--print("and the active player owns it");
				-- Add perk's values to the total
				BuildingAffinityYields.Supremacy	= BuildingAffinityYields.Supremacy	+ ItemAffinYieldInfo.FlatSupremacy;
				BuildingAffinityYields.Harmony		= BuildingAffinityYields.Harmony	+ ItemAffinYieldInfo.FlatHarmony;
				BuildingAffinityYields.Purity		= BuildingAffinityYields.Purity		+ ItemAffinYieldInfo.FlatPurity;
			end
		end
	end
		
	--print("Building yields " .. BuildingAffinityYields.Supremacy .. "S " .. BuildingAffinityYields.Harmony .. "H " .. BuildingAffinityYields.Purity .. "P");
	return BuildingAffinityYields;
end


-- Calculates citywide affinity yield-modifiers from a given building type, and returns them as a table
function GetAffinityYieldModifiersFromBuilding(BuildingType)
	local BuildingYieldModifiers = { Supremacy = 0, Harmony = 0, Purity = 0 };
	--print("getting perk modifiers for " .. BuildingType);

	-- Base percentage modifiers for this building type
	BuildingYieldModifiers.Supremacy = CachedBuildingAffinYieldMods[BuildingType].Supremacy;
	BuildingYieldModifiers.Harmony = CachedBuildingAffinYieldMods[BuildingType].Harmony;
	BuildingYieldModifiers.Purity = CachedBuildingAffinYieldMods[BuildingType].Purity;

	-- Extra percentage modifiers for this building type, from Perks etc.

	-- iterate through all perks which affect affinity yields
	for perk, ItemAffinYieldInfo in pairs(CachedAffinYieldBuildingPerks) do
		
		-- does this perk affect this building?
		if ItemAffinYieldInfo.Type == BuildingType then
			--print("A perk exists which affects this building");

			-- does the player have this perk?
			local pActivePlayer = Players[Game.GetActivePlayer()];
			

			if pActivePlayer:HasPerk(ItemAffinYieldInfo.PerkID) then
				--print("and the active player owns it");
				-- Add perk's values to the total
				BuildingYieldModifiers.Supremacy	= BuildingYieldModifiers.Supremacy	+ ItemAffinYieldInfo.PercentSupremacy;
				BuildingYieldModifiers.Harmony		= BuildingYieldModifiers.Harmony	+ ItemAffinYieldInfo.PercentHarmony;
				BuildingYieldModifiers.Purity		= BuildingYieldModifiers.Purity		+ ItemAffinYieldInfo.PercentPurity;
			end
		end
	end

	--print("Building yields " .. BuildingYieldModifiers.Supremacy .. "%S " .. BuildingYieldModifiers.Harmony .. "%H " .. BuildingYieldModifiers.Purity .. "%P");
	return BuildingYieldModifiers;
end



-- TO DO: Delete national wonders from a city if the headquarters in it are lost (due to the capital city being captured/retaken).