-- Affinity_Yields_AI_Helper
-- Author: HandyVac
-- DateCreated: 8/20/2015 4:42:36 PM
--------------------------------------------------------------
-- The AI is not good at choosing buildings/improvements to pursue a specific affinity. Left to itself it tends to level up all three affinities evenly.
-- This script intends to steer the AI towards improving one dominant affinity more than the other two.

print("#### Loaded Affinity As Yields AI Helper Script ####");

local HarmonyID = GameInfo.Affinity_Types["AFFINITY_TYPE_HARMONY"].ID
local PurityID = GameInfo.Affinity_Types["AFFINITY_TYPE_PURITY"].ID
local SupremacyID = GameInfo.Affinity_Types["AFFINITY_TYPE_SUPREMACY"].ID

local PCW_ClassID = GameInfo.BuildingClasses["BUILDINGCLASS_PURITY_COLONIAL_WONDER"].ID;
local HCW_ClassID = GameInfo.BuildingClasses["BUILDINGCLASS_HARMONY_COLONIAL_WONDER"].ID;
local SCW_ClassID = GameInfo.BuildingClasses["BUILDINGCLASS_SUPREMACY_COLONIAL_WONDER"].ID;
local CWOffice_ClassID = GameInfo.BuildingClasses["BUILDINGCLASS_COLONIAL_OFFICE"].ID;

local PIW_ClassID = GameInfo.BuildingClasses["BUILDINGCLASS_PURITY_IMPERIAL_WONDER"].ID;
local HIW_ClassID = GameInfo.BuildingClasses["BUILDINGCLASS_HARMONY_IMPERIAL_WONDER"].ID;
local SIW_ClassID = GameInfo.BuildingClasses["BUILDINGCLASS_SUPREMACY_IMPERIAL_WONDER"].ID;
local IWOffice_ClassID = GameInfo.BuildingClasses["BUILDINGCLASS_IMPERIAL_OFFICE"].ID;

function SteerAITowardAffinityWonders(CurrentPlayerID)
	local pPlayer = Players[CurrentPlayerID];

	print("Running AAY AI Helper for player " .. CurrentPlayerID .. " : " .. pPlayer:GetName() );
	-- early out if this is the human player
	if pPlayer:IsHuman() then
		print("...this is the human player. Doing nothing.");
		return false;
	end

	local iDominantAffinity_ID = pPlayer:GetDominantAffinityType();
	print(pPlayer:GetAffinityLevel(iDominantAffinity_ID) .. " levels in dominant affinity");

	-- COLONIAL WONDERS

	-- has the player reached affinity level 3?
	if pPlayer:GetAffinityLevel(iDominantAffinity_ID) >= 3 then 
		print("Player " .. CurrentPlayerID .. " has reached level 3 in " .. GameInfo.Affinity_Types[iDominantAffinity_ID].Type);

		-- does the palyer own the colonial wonder for this affinity?
		if pPlayer:GetBuildingClassCount( ColonialWonder(iDominantAffinity_ID) ) > 0 then
	
			print("...and owns a " .. GameInfo.BuildingClasses[ColonialWonder(iDominantAffinity_ID)].Type .. " :)") 
		else
			print("...but does not have a " .. GameInfo.BuildingClasses[ColonialWonder(iDominantAffinity_ID)].Type)

			-- if player has a different colonial wonder, delete it.
			if pPlayer:GetBuildingClassCount( ColonialWonder(HarmonyID) ) > 0 then
				print("...they have a " .. GameInfo.BuildingClasses[ColonialWonder(HarmonyID)].Type .. " instead. Deleting it");
				pPlayer:RemoveBuildingClass( ColonialWonder(HarmonyID) );
				pPlayer:RemoveBuildingClass( CWOffice_ClassID );
			end

			if pPlayer:GetBuildingClassCount( ColonialWonder(PurityID) ) > 0 then
				print("...they have a " .. GameInfo.BuildingClasses[ColonialWonder(PurityID)].Type .. " instead. Deleting it");
				pPlayer:RemoveBuildingClass( ColonialWonder(PurityID) );
				pPlayer:RemoveBuildingClass( CWOffice_ClassID );
			end

			if pPlayer:GetBuildingClassCount( ColonialWonder(SupremacyID) ) > 0 then
				print("...they have a " .. GameInfo.BuildingClasses[ColonialWonder(SupremacyID)].Type .. " instead. Deleting it");
				pPlayer:RemoveBuildingClass( ColonialWonder(SupremacyID) );
				pPlayer:RemoveBuildingClass( CWOffice_ClassID );
			end
			
			-- if player has the tech to construct the proper wonder, give them one for free
			local sCW_BuildingType = GameInfo.BuildingClasses[ColonialWonder(iDominantAffinity_ID)].DefaultBuilding;
			local iCW_BuildingType_ID = GameInfo.Buildings[sCW_BuildingType].ID;
			local sTechToBuildCW = GameInfo.Buildings[sCW_BuildingType].PrereqTech;
			local iTechToBuildCW_ID = GameInfo.Technologies[sTechToBuildCW].ID;

			if pPlayer:HasTech(iTechToBuildCW_ID) then
				-- if player has tech to build colonial wonder, give them one for free
				print("...adding a free " .. sCW_BuildingType .. " in their capital");
				local pCity = pPlayer:GetCapitalCity();
				pCity:SetNumRealBuilding( iCW_BuildingType_ID, 1 );
			else
				-- if player doesn't have tech, force them to start researching it.
				print("...can't build " .. sCW_BuildingType .. " without " .. sTechToBuildCW);
				if pPlayer:GetCurrentResearch() == iCW_BuildingType_ID then
					print("...currently researching " .. sTechToBuildCW);
				else
					print("...clearing research queue");
					pPlayer:ClearResearchQueue();
					print("...pushing " .. sTechToBuildCW .." to research queue");
					pPlayer:PushResearch(iTechToBuildCW_ID)
				end
			end		
		end
	end


	-- IMPERIAL WONDERS 

	-- has the player reached affinity level 6?
	if pPlayer:GetAffinityLevel(iDominantAffinity_ID) >= 6 then 
		print("Player " .. CurrentPlayerID .. " has reached level 6 in " .. GameInfo.Affinity_Types[iDominantAffinity_ID].Type);
		-- does the palyer own the imperial wonder for this affinity?
		if pPlayer:GetBuildingClassCount( ImperialWonder(iDominantAffinity_ID) ) > 0 then
			print("...and owns a " .. GameInfo.BuildingClasses[ImperialWonder(iDominantAffinity_ID)].Type .. " :)") 
		else
			print("...but does not have a " .. GameInfo.BuildingClasses[ImperialWonder(iDominantAffinity_ID)].Type)

			-- if player has a different imperial wonder, delete it.
			if pPlayer:GetBuildingClassCount( ImperialWonder(HarmonyID) ) > 0 then
				print("...they have a " .. GameInfo.BuildingClasses[ImperialWonder(HarmonyID)].Type .. " instead. Deleting it");
				pPlayer:RemoveBuildingClass( ImperialWonder(HarmonyID) );
				pPlayer:RemoveBuildingClass( IWOffice_ClassID );
			end

			if pPlayer:GetBuildingClassCount( ImperialWonder(PurityID) ) > 0 then
				print("...they have a " .. GameInfo.BuildingClasses[ImperialWonder(PurityID)].Type .. " instead. Deleting it");
				pPlayer:RemoveBuildingClass( ImperialWonder(PurityID) );
				pPlayer:RemoveBuildingClass( IWOffice_ClassID );
			end

			if pPlayer:GetBuildingClassCount( ImperialWonder(SupremacyID) ) > 0 then
				print("...they have a " .. GameInfo.BuildingClasses[ImperialWonder(SupremacyID)].Type .. " instead. Deleting it");
				pPlayer:RemoveBuildingClass( ImperialWonder(SupremacyID) );
				pPlayer:RemoveBuildingClass( IWOffice_ClassID );
			end
			
			-- if player has the tech to construct the proper wonder, give them one for free
			local sIW_BuildingType = GameInfo.BuildingClasses[ImperialWonder(iDominantAffinity_ID)].DefaultBuilding;
			local iIW_BuildingType_ID = GameInfo.Buildings[sIW_BuildingType].ID;
			local sTechToBuildIW = GameInfo.Buildings[sIW_BuildingType].PrereqTech;
			local iTechToBuildIW_ID = GameInfo.Technologies[sTechToBuildIW].ID;

			if pPlayer:HasTech(iTechToBuildIW_ID) then
				-- if player has tech to build imperial wonder, give them one for free
				print("...adding a free " .. sIW_BuildingType .. " in their capital");
				local pCity = pPlayer:GetCapitalCity();
				pCity:SetNumRealBuilding( iIW_BuildingType_ID, 1 );
			else
				-- if player doesn't have tech, force them to start researching it.
				print("...can't build " .. sIW_BuildingType .. " without " .. sTechToBuildIW);
				if pPlayer:GetCurrentResearch() == iIW_BuildingType_ID then
					print("...currently researching " .. sTechToBuildIW);
				else
					print("...clearing research queue");
					pPlayer:ClearResearchQueue();
					print("...pushing " .. sTechToBuildIW .." to research queue");
					pPlayer:PushResearch(iTechToBuildIW_ID)
				end
			end	


		end
	end
end

-- call this at start of every player's turn
GameEvents.PlayerDoTurn.Add(SteerAITowardAffinityWonders);

function ColonialWonder(AffinityTypeID)

	if AffinityTypeID == HarmonyID then
		return HCW_ClassID;
	end
	if AffinityTypeID == PurityID then
		return PCW_ClassID;
	end
	if AffinityTypeID == SupremacyID then
		return SCW_ClassID;
	end

end

function ImperialWonder(AffinityTypeID)

	if AffinityTypeID == HarmonyID then
		return HIW_ClassID;
	end
	if AffinityTypeID == PurityID then
		return PIW_ClassID;
	end
	if AffinityTypeID == SupremacyID then
		return SIW_ClassID;
	end

end